# $Id: client.pl,v 1.1.1.1 1997/02/05 23:02:14 fox Exp $
#
#  Code to read client info from pythia.clients and determine client type given
#  HTTP headers.

=head1 NAME

client.pl - manages client device characteristics

=head1 SYNOPSIS

    &client::read_clients();
    $prefs_hash_ref = &client::client_prefs_from_headers($hdrs_hash_ref);

=head1 DESCRIPTION

The I<read_clients> call reads F<pythia.clients> for information about
possible client device types and how to identify them from their HTTP
headers.  See that file for details.

I<client_prefs_from_headers> returns a reference to a hash of the client
characteristics as specified in F<pythia.clients>, given a reference to
a hash whose keys are HTTP header names and whose values are the
contents (no newlines) of those headers.  The header names should be in
canonical form: the first letter and every letter following a hyphen
should be capitalized.

=cut
    
package client;

@Hdrs = @Regexps = @Clients = ();

sub read_clients {
    my $file = shift || "pythia.clients";
    open(F,$file) or die "client: no clients file $file\n";
    my $cli;
    
    while (<F>) {
        next if /^\s*$/;
        next if /^#/;
        die "Bad entry: $_" unless
            /^(\S+)\s+(\S+)\s+(.*)/;
        push(@Hdrs, lc $1);
        push(@Regexps, $cli = lc $2);
        my ($a,%a);
        foreach $a (split (/\s+/, $3)) {
            die "Bad attrib for $_: $a"
                unless $a =~ /(\S+)=(\S+)/;
            $a{lc $1} = $2;
        }
        push(@Clients, \%a);
        &main::debug('client', "Client $cli");
    }
    close F;
}

sub client_prefs_from_headers {
    my $hdrs = shift;
    my $success = 0;
    
    for ($i=0; $i <= $#Clients; $i++) {
        $success = 1,last if $hdrs->header($Hdrs[$i]) =~ /$Regexps[$i]/i;
    }

    &main::log("*** client: no client matching headers")
        unless $success;

    return $Clients[$i];
}

1;
